/* ------------------------------------------------------------------ */
/*      item            : EntityCommand.hxx
        generated by    : Autobuild (abuild)
        date            : Fri 07 Mar 2025
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : EntityCommand.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : Control entity state machine
        copyright       : (c) 2016-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated EntityCommand.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#ifndef NESTED_DCO
#define NESTED_DCO
#define MASTER_DCO_EntityCommand
#endif
#include <SimulationState.hxx>
#ifdef MASTER_DCO_EntityCommand
#undef MASTER_DCO_EntityCommand
#undef NESTED_DCO
#endif

namespace dueca {
 
/** command to change DUSIME module state
    */
struct EntityCommand
{
  /** typedef for internal reference */
  typedef EntityCommand __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** EntitycommandType, for controlling simulation and hardware modules

    this is used for DUSIME modules
 */
  enum EntityCommandType {
    /** this command sends a new target state */
    NewState,
    /** prepare memory etc for taking a snapshot */
    PrepareSnapshot,
    /** send the snapshot */
    SendSnapshot,
    /** send a snapshot that is the result from initial state calculation
        note that the terms initial state and trim state are sometimes used
        intermittently */
    SendIncoSnapshot,
    /** confirm the present state */
    ConfirmState
  };

  /** type of this command */
  EntityCommandType command;

  /** new state for the DUSIME modules */
  SimulationState new_state;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  EntityCommand();

  /** Constructor with arguments */
  EntityCommand(
        const EntityCommandType& command,
        const SimulationState& new_state);

  /** copy constructor. */
  EntityCommand(const EntityCommand& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an EntityCommand from amorphous storage. */
  EntityCommand(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~EntityCommand();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** accompanying placement operator "new" */
  static void* operator new(size_t size, void* mem);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, EntityCommand*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the EntityCommand into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the EntityCommand into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const EntityCommand& ref) const;

  /** unpacks the EntityCommand from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for EntityCommand
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const EntityCommand& o) const;

  /** Test for inequality. */
  inline bool operator != (const EntityCommand& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  EntityCommand& operator=(const EntityCommand& o);

  /** prints the EntityCommand to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::EntityCommand& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::EntityCommand& o,
                         const dueca::EntityCommand& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::EntityCommand& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::EntityCommand& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const EntityCommand::EntityCommandType &o);
  void readFromString(EntityCommand::EntityCommandType &o, const std::string& s);
  void getFirst(EntityCommand::EntityCommandType &o);
  bool getNext(EntityCommand::EntityCommandType &o);

  /** This function retrieves the classname of an EntityCommand::EntityCommandType */
  template <>
  const char* getclassname<EntityCommand::EntityCommandType>();
  /** This function retrieves a string with int rep EntityCommand::EntityCommandType */
  template <>
  const char* getenumintrep<EntityCommand::EntityCommandType>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::EntityCommand::EntityCommandType &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::EntityCommand::EntityCommandType &o);
#endif

PRINT_NS_START;
/** Print an object of type EntityCommand::EntityCommandType */
inline std::ostream&
operator << (std::ostream& s, const dueca::EntityCommand::EntityCommandType& o)
{ return s << dueca::getString(o); }
/** Read an object of type EntityCommand::EntityCommandType */
inline std::istream&
/** Read an enum of type EntityCommand::EntityCommandType */
operator >> (std::istream& s, dueca::EntityCommand::EntityCommandType& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::EntityCommand& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::EntityCommand>();
template <>
inline const char* dueca::getclassname<const dueca::EntityCommand>()
{ return dueca::getclassname<dueca::EntityCommand>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    EntityCommand is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::EntityCommand> : public dco_isnested { };
template <>
struct dco_nested<dueca::EntityCommand::EntityCommandType> : public dco_isenum { };
};

#endif


