/* ------------------------------------------------------------------ */
/*      item            : NetCapacityLog.hxx
        generated by    : Autobuild (abuild)
        date            : Fri 07 Mar 2025
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : NetCapacityLog.dco
        made by         : Rene' van Paassen
        date            : 170205
        description     : Peer information replicator nodes
        documentation   : DUECA_API
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated NetCapacityLog.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#ifndef NESTED_DCO
#define NESTED_DCO
#define MASTER_DCO_NetCapacityLog
#endif
#include <inttypes.h>
#include <dueca/SimTime.hxx>
#include <dueca/fixvector.hxx>
#ifdef MASTER_DCO_NetCapacityLog
#undef MASTER_DCO_NetCapacityLog
#undef NESTED_DCO
#endif

namespace dueca {
 
/** Network capacity use information.
   
    Provides, per participating node, a coarse histogram of message size
    split into regular data size, and total size (including fill data)
    */
struct NetCapacityLog
{
  /** typedef for internal reference */
  typedef NetCapacityLog __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** node id for which this capacity info is collected */
  uint16_t node_id;

  /** number of cycles counted */
  uint16_t n_points;

  /** standard message capacity histogram */
  fixvector<10,uint16_t> regular;

  /** total fill capacity histogram */
  fixvector<10,uint16_t> total;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  NetCapacityLog();

  /** Constructor with arguments */
  NetCapacityLog(
        const uint16_t& node_id,
        const uint16_t& n_points,
        const fixvector<10,uint16_t>& regular,
        const fixvector<10,uint16_t>& total);

  /** copy constructor. */
  NetCapacityLog(const NetCapacityLog& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an NetCapacityLog from amorphous storage. */
  NetCapacityLog(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~NetCapacityLog();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** accompanying placement operator "new" */
  static void* operator new(size_t size, void* mem);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, NetCapacityLog*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the NetCapacityLog into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the NetCapacityLog into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const NetCapacityLog& ref) const;

  /** unpacks the NetCapacityLog from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for NetCapacityLog
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const NetCapacityLog& o) const;

  /** Test for inequality. */
  inline bool operator != (const NetCapacityLog& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  NetCapacityLog& operator=(const NetCapacityLog& o);

  /** prints the NetCapacityLog to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "NetCapacityLogExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::NetCapacityLog& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::NetCapacityLog& o,
                         const dueca::NetCapacityLog& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::NetCapacityLog& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::NetCapacityLog& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::NetCapacityLog& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::NetCapacityLog>();
template <>
inline const char* dueca::getclassname<const dueca::NetCapacityLog>()
{ return dueca::getclassname<dueca::NetCapacityLog>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    NetCapacityLog is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::NetCapacityLog> : public dco_isnested { };
};

#endif


