/* ------------------------------------------------------------------ */
/*      item            : NetTimingLog.hxx
        generated by    : Autobuild (abuild)
        date            : Fri 07 Mar 2025
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : NetTimingLog.dco
        made by         : Rene' van Paassen
        date            : 170205
        description     : Peer information replicator nodes
        documentation   : DUECA_API
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated NetTimingLog.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#ifndef NESTED_DCO
#define NESTED_DCO
#define MASTER_DCO_NetTimingLog
#endif
#include <inttypes.h>
#include <dueca/SimTime.hxx>
#include <inttypes.h>
#include <fixvector.hxx>
#ifdef MASTER_DCO_NetTimingLog
#undef MASTER_DCO_NetTimingLog
#undef NESTED_DCO
#endif

namespace dueca {
 
/** Timing information on network use
   
    This provides a histogram view of the network communication time cycle.
    Timing is logged for a number of cycles, giving a histogram of times
    */
struct NetTimingLog
{
  /** typedef for internal reference */
  typedef NetTimingLog __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** number of items logged */
  uint16_t n_points;

  /** maximum in [us] */
  uint32_t t_max;

  /** estimate for message set-up */
  float net_permessage;

  /** estimate for byte transmission time */
  float net_perbyte;

  /** standard capacity */
  fixvector<20,uint16_t> times;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  NetTimingLog();

  /** Constructor with arguments */
  NetTimingLog(
        const uint16_t& n_points,
        const uint32_t& t_max,
        const float& net_permessage,
        const float& net_perbyte,
        const fixvector<20,uint16_t>& times);

  /** copy constructor. */
  NetTimingLog(const NetTimingLog& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an NetTimingLog from amorphous storage. */
  NetTimingLog(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~NetTimingLog();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** accompanying placement operator "new" */
  static void* operator new(size_t size, void* mem);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, NetTimingLog*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the NetTimingLog into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the NetTimingLog into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const NetTimingLog& ref) const;

  /** unpacks the NetTimingLog from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for NetTimingLog
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const NetTimingLog& o) const;

  /** Test for inequality. */
  inline bool operator != (const NetTimingLog& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  NetTimingLog& operator=(const NetTimingLog& o);

  /** prints the NetTimingLog to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "NetTimingLogExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::NetTimingLog& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::NetTimingLog& o,
                         const dueca::NetTimingLog& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::NetTimingLog& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::NetTimingLog& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::NetTimingLog& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::NetTimingLog>();
template <>
inline const char* dueca::getclassname<const dueca::NetTimingLog>()
{ return dueca::getclassname<dueca::NetTimingLog>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    NetTimingLog is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::NetTimingLog> : public dco_isnested { };
};

#endif


